import argparse
import collections

import urwid

import aws_top.signals
from aws_top import aws
from aws_top.popup import GenerousPopUpLauncher, RegionSelectorDialog, ServiceSelectorDialog
from aws_top.window import StatusWindow, OptionWindow, Option, Ec2Window, S3Window


class AwsTop:
    def __init__(self, access_key=None, secret_key=None, session_token=None, region=None):
        if access_key and secret_key or session_token:
            aws.set_credentials(access_key, secret_key, session_token)

        if region:
            aws.set_region(region)

        services = collections.OrderedDict({
            'EC2': Ec2Window,
            'S3': S3Window,
            'DynamoDB': None
        })

        def exit_main_loop():
            raise urwid.ExitMainLoop()

        def change_service(w, service):
            self.service_window.original_widget = services[service]()

        popup_anchor = urwid.BoxAdapter(urwid.SolidFill(), 0)
        region_selector = RegionSelectorDialog()
        service_selector = ServiceSelectorDialog(list(services.keys()))
        region_popup_launcher = GenerousPopUpLauncher(popup_anchor, region_selector)
        service_popup_launcher = GenerousPopUpLauncher(popup_anchor, service_selector)

        self.status_window = StatusWindow()
        self.service_window = urwid.Filler(list(services.values())[0](), valign=urwid.TOP)
        self.option_window = OptionWindow(
            [
                Option('Help', lambda: None),
                Option('Region', lambda: region_popup_launcher.open_pop_up()),
                Option('Service', lambda: service_popup_launcher.open_pop_up()),
                Option('Exit', exit_main_loop)
            ]
        )

        urwid.connect_signal(service_selector, aws_top.signals.SERVICE_CHANGE, change_service)

        # Layout:
        # - invisible PopUp launchers
        # - Status
        # - Main
        # - Options
        self.main_win = urwid.Pile(
            [
                ('pack', urwid.Pile([region_popup_launcher, service_popup_launcher])),
                ('pack', urwid.BoxAdapter(self.status_window, 1)),
                self.service_window,
                ('pack', urwid.BoxAdapter(self.option_window, 1))
            ],
            focus_item=3
        )

        palette = [
            ('error', 'dark red', ''),
            ('warn', 'yellow', ''),
            ('f_key', 'bold,white', 'default'),
            ('options_bg', 'bold', 'dark blue'),
            ('bold', 'bold', ''),
            ('popbg', 'white', 'dark blue')
        ]

        self.__loop = urwid.MainLoop(
            self.main_win,
            palette=palette,
            pop_ups=True
        )
        self.__loop.screen.set_terminal_properties(colors=256)

    def update_ui(self, loop, user_data=None):
        self.status_window.update()
        self.service_window.original_widget.update()

        self.__loop.draw_screen()
        self.__loop.set_alarm_in(1, self.update_ui)

    def run(self):
        self.__loop.set_alarm_in(1, self.update_ui)
        self.__loop.run()


def main():
    argparser = argparse.ArgumentParser()

    argparser.add_argument('-a', '--access-key')
    argparser.add_argument('-s', '--secret-key')
    argparser.add_argument('-S', '--session-token')
    argparser.add_argument('-r', '--region')

    args = argparser.parse_args()

    AwsTop(
        args.access_key,
        args.secret_key,
        args.session_token,
        args.region
    ).run()


if __name__ == '__main__':
    main()



from setuptools import setup

setup(
    name='aws-top',
    version='0.3',
    packages=['aws_top'],
    entry_points={
        'console_scripts': [
            'aws-top = aws_top.__main__:main',
        ],
    },
    install_requires=['boto3==1.7.20', 'urwid==2.0.1'],
    url='https://github.com/brennerm/aws-top',
    license='MIT',
    author='brennerm',
    author_email='xamrennerb@gmail.com',
    description='CLI Dashboard for AWS services'
)



#!/usr/bin/env python3
import botocore.exceptions
import time

import urwid

import aws_top.aws


class WindowManager:
    ONE_EIGHT = '&#9601;'
    ONE_QUARTER = '&#9602;'
    THREE_EIGHTS = '&#9603;'
    HALF = '&#9604;'
    FIVE_EIGHTS = '&#9605;'
    THREE_QUARTERS = '&#9606;'
    SEVEN_EIGHTS = '&#9607;'
    FULL = '&#9608;'


class StatusWindow(urwid.Widget):
    _sizing = frozenset([urwid.BOX])

    def __init__(self):
        super(StatusWindow, self).__init__()
        self.__time = None
        self.__user = aws_top.aws.get_user()
        self.__region = None

        self.update()

    def update(self):
        self.__time = time.strftime(
            '%b %d %Y %H:%M:%S'
        )
        self.__region = aws_top.aws.get_region()
        self._invalidate()

    def render(self, size, focus=False):
        body = [
            urwid.Text('Logged in as: {}'.format(self.__user)),
            urwid.Text('Region: {}'.format(self.__region), align=urwid.CENTER),
            urwid.Text('Time: {}'.format(self.__time), align=urwid.RIGHT)
        ]

        return urwid.Columns(body).render((size[0],))


class Ec2Window(urwid.Widget):
    _sizing = frozenset([urwid.FLOW])

    def __init__(self):
        self.__instances = []
        self.__error = None
        self.update()

    def update(self):
        try:
            self.__instances = aws_top.aws.Ec2().get_all_instances()
            self.__error = None
        except botocore.exceptions.ClientError as ex:
            self.__error = ex.response
        self._invalidate()

    def rows(self, size, focus=False):
        return 1 + max(len(self.__instances), 1)

    def render(self, size, focus=False):
        if self.__error:
            return urwid.Text(('error', self.__error), align=urwid.CENTER).render(size)

        body = []
        header = [
            urwid.Text('ID'),
            urwid.Text('Name', align=urwid.LEFT),
            urwid.Text('State', align=urwid.LEFT),
            urwid.Text('Type', align=urwid.LEFT),
            urwid.Text('AZ', align=urwid.LEFT)
        ]

        body.append(
            urwid.AttrMap(urwid.Columns(header), 'bold')
        )

        if len(self.__instances) == 0:
            body.append(urwid.Text(('warn', 'No available EC2 instances in this region.'), align=urwid.CENTER))

        for i, instance in enumerate(self.__instances, 1):
            body.append(
                urwid.Columns(
                    [
                        urwid.Text(instance.id),
                        urwid.Text(instance.name or 'N/A'),
                        urwid.Text(instance.state),
                        urwid.Text(instance.instance_type),
                        urwid.Text(instance.az)
                    ]
                )
            )

        return urwid.Pile(body).render(size)


class S3Window(urwid.Widget):
    _sizing = frozenset([urwid.FLOW])

    def __init__(self):
        self.__buckets = []
        self.__error = None
        self.update()

    def update(self):
        try:
            self.__buckets = aws_top.aws.S3().get_all_buckets()
            self.__error = None
        except botocore.exceptions.ClientError as ex:
            self.__error = ex.response
        self._invalidate()

    def rows(self, size, focus=False):
        return 1 + max(len(self.__buckets), 1)

    def render(self, size, focus=False):
        if self.__error:
            return urwid.Text(('error', self.__error), align=urwid.CENTER).render(size)

        body = []

        header = [
            urwid.Text('Name'),
            urwid.Text('Creation Date', align=urwid.LEFT),
        ]

        body.append(
            urwid.AttrMap(urwid.Columns(header), 'bold')
        )

        if len(self.__buckets) == 0:
            body.append(urwid.Text(('warn', 'No available S3 buckets in this region.'), align=urwid.CENTER))

        for i, bucket in enumerate(self.__buckets, 1):
            body.append(
                urwid.Columns(
                    [
                        urwid.Text(bucket.name),
                        urwid.Text(bucket.creation_date.strftime("%Y-%m-%d %H:%M:%S"))
                    ]
                )
            )

        return urwid.Pile(body).render(size)


class OptionWindow(urwid.Widget):
    _sizing = frozenset([urwid.BOX])
    _selectable = True

    def __init__(self, options=None):
        self.__options = options or []

    def keypress(self, size, key):
        for i in range(len(self.__options)):
            if key == 'f' + str(i + 1):
                self.__options[i].trigger()
                return

        return key

    def render(self, size, focus=False):
        body = []

        for i, option in enumerate(self.__options, start=1):
            body.append(
                urwid.Text(
                    [
                        ('f_key', "F" + str(i)),
                        ('options_bg', option.text)
                    ]
                )
            )

        return urwid.AttrMap(
            urwid.Columns(
                body
            ),
            'options_bg'
        ).render((size[0],))

    def set_options(self, options):
        if len(options) > 12:
            raise ValueError('Number of options exceeds 12')

        self.__options = options
        self._invalidate()


class Option:
    def __init__(self, text, action):
        self.__text = text
        self._action = action

    def __len__(self):
        return len(self.text)

    @property
    def text(self):
        return self.__text

    @text.setter
    def text(self, value):
        self.__text = value

    def trigger(self):
        self._action()


class ToggableOption(Option):
    def __init__(self, texts, action):
        self.__text1 = texts[0]
        self.__text2 = texts[1]
        self.__flag = True

        super(ToggableOption, self).__init__(self.__text1, action)

    @property
    def text(self):
        return self.__text1 if self.__flag else self.__text2

    @text.setter
    def text(self, value):
        return

    def __toggle(self):
        self.__flag = not self.__flag

    def trigger(self):
        self._action()
        self.__toggle()
